
from django.shortcuts import redirect, render
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _
from django.http.response import HttpResponse

from contacts.forms import FeedbackForm, ReturnCallForm, ErrorMessageForm
from contacts.lib import (
    send_new_feedback_notification, send_new_return_call_notification,
    send_new_error_message_notification)


def index(request):

    data = request.POST
    user = request.user

    feedback_form_params = {'data': data if data.get('feedback') else None}
    return_call_form_params = {'data': data if data.get('call') else None}

    if user.is_authenticated():
        mobile = user.profile.mobile if hasattr(user, 'profile') else ''

        feedback_form_params['initial'] = {
            'name': user.get_full_name(),
            'email': user.email,
            'mobile': mobile
        }
        return_call_form_params['initial'] = {
            'mobile': mobile
        }

    feedback_form = FeedbackForm(**feedback_form_params)
    return_call_form = ReturnCallForm(**return_call_form_params)

    if data and data.get('feedback'):

        if feedback_form.is_valid():

            feedback = feedback_form.save(commit=False)

            if user.is_authenticated():
                feedback.user = user

            feedback.save()

            messages.success(request, _('Message was successfully sent'))

            send_new_feedback_notification(feedback)

            return redirect(request.path)

        else:
            messages.error(
                request, _('Please fix the errors in the feedback form'))

    if data and data.get('call'):

        if return_call_form.is_valid():

            return_call = return_call_form.save(commit=False)

            if user.is_authenticated():
                return_call.user = user

            return_call.save()

            messages.success(request, _('Call request was successfully sent'))

            send_new_return_call_notification(return_call)

            return redirect(request.path)

        else:
            messages.error(
                request, _('Please fix the errors in the return call form'))

    context = {
        'feedback_form': feedback_form,
        'return_call_form': return_call_form
    }

    return render(request, 'contacts/index.html', context)


def create_return_call(request):

    user = request.user

    form_params = {'data': request.POST or None}

    if user.is_authenticated():
        mobile = user.profile.mobile if hasattr(user, 'profile') else ''
        form_params['initial'] = {'mobile': mobile}

    form = ReturnCallForm(**form_params)

    if request.method == 'POST':

        if form.is_valid():

            return_call = form.save(commit=False)

            if user.is_authenticated():
                return_call.user = user

            return_call.save()

            send_new_return_call_notification(return_call)

            return HttpResponse(_('Call request was successfully sent'))

        else:
            return render(
                request, 'contacts/return_call/form.html',
                {'form': form}, status=403)

    return render(request, 'contacts/return_call/modal.html', {'form': form})


def create_error_message(request):

    user = request.user

    form_params = {'data': request.POST or None}

    if user.is_authenticated():
        mobile = user.profile.mobile if hasattr(user, 'profile') else ''
        form_params['initial'] = {
            'name': user.get_full_name(),
            'email': user.email,
            'mobile': mobile
        }

    form = ErrorMessageForm(**form_params)

    if request.method == 'POST':

        if form.is_valid():

            error_message = form.save(commit=False)

            if user.is_authenticated():
                error_message.user = user

            error_message.url = request.GET.get('url', '/')

            error_message.save()

            send_new_error_message_notification(error_message)

            return HttpResponse(_('Error message was successfully sent'))

        else:
            return render(
                request, 'contacts/error_message/form.html',
                {'form': form}, status=403)

    return render(request, 'contacts/error_message/modal.html', {'form': form})


def create_feedback(request):

    user = request.user

    form_params = {'data': request.POST or None}

    if user.is_authenticated():
        mobile = user.profile.mobile if hasattr(user, 'profile') else ''
        form_params['initial'] = {
            'name': user.get_full_name(),
            'email': user.email,
            'mobile': mobile
        }

    form = FeedbackForm(**form_params)

    if request.method == 'POST':

        if form.is_valid():

            feedback = form.save(commit=False)

            if user.is_authenticated():
                feedback.user = user

            feedback.save()

            send_new_feedback_notification(feedback)

            return HttpResponse(_('Message was successfully sent'))

        else:
            return render(
                request, 'contacts/feedback/form.html',
                {'form': form}, status=403)

    return render(request, 'contacts/feedback/modal.html', {'form': form})



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-02-01 17:12
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Feedback',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('subject', models.CharField(max_length=255, verbose_name='Subject')),
                ('mobile', models.CharField(blank=True, max_length=255, null=True, verbose_name='Mobile')),
                ('email', models.EmailField(blank=True, max_length=255, null=True, verbose_name='Email')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('text', models.TextField(max_length=1000, verbose_name='Message text')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='feedbacks', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Feedback message',
                'verbose_name_plural': 'Feedback messages',
            },
        ),
        migrations.CreateModel(
            name='ReturnCall',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('answer_time', models.PositiveIntegerField(choices=[(1, 'As fast as possible'), (2, 'In fixed time')], default=1, verbose_name='Answer time')),
                ('answer_start_time', models.TimeField(blank=True, default=b'08:00', null=True, verbose_name='Start time')),
                ('answer_end_time', models.TimeField(blank=True, default=b'08:30', null=True, verbose_name='End time')),
                ('mobile', models.CharField(max_length=21, verbose_name='Mobile')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('comment', models.TextField(blank=True, max_length=1000, verbose_name='Comment')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='return_calls', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Return call',
                'verbose_name_plural': 'Return calls',
            },
        ),
    ]




from django.contrib import admin

from contacts.models import Feedback, ReturnCall, ErrorMessage


class FeedbackAdmin(admin.ModelAdmin):
    list_display = (
        'subject', 'id', 'user', 'name', 'mobile', 'email', 'date_created',
    )
    search_fields = ('name', 'subject', 'mobile', 'email', 'text', )


class ReturnCallAdmin(admin.ModelAdmin):
    list_display = (
        'mobile', 'id', 'get_answer_time_display', 'user', 'answer_start_time',
        'answer_end_time', 'date_created',
    )
    list_filter = ('answer_time',)
    search_fields = ('mobile', )


class ErrorMessageAdmin(admin.ModelAdmin):
    list_display = ('id', 'user', 'name', 'mobile', 'email', 'date_created', )
    search_fields = ('name', 'mobile', 'email', 'text', )


admin.site.register(ReturnCall, ReturnCallAdmin)
admin.site.register(ErrorMessage, ErrorMessageAdmin)
admin.site.register(Feedback, FeedbackAdmin)




from django.conf.urls import url

from contacts import views


urlpatterns = [

    url(r'^$', views.index, name='index'),

    url(r'^return-call/$', views.create_return_call, name='return-call'),

    url(r'^error-message/$', views.create_error_message, name='error-message'),

    url(r'^feedback/$', views.create_feedback, name='feedback'),

]




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class ContactsAppConfig(AppConfig):
    name = 'contacts'
    verbose_name = _('Contacts')


default_app_config = 'contacts.ContactsAppConfig'

__version__ = '3.3'




from django.apps import apps
from django.core.mail import mail_managers
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from django.contrib.sites.models import Site


def _send_sms(template, context):
    if apps.is_installed('turbosms'):
        from turbosms.lib import send_sms_from_template
        send_sms_from_template(template, context)


def send_new_feedback_notification(feedback):

    subject = _('New feedback message #') + str(feedback.id)

    email_template = 'contacts/feedback/email.html'

    context = {
        'feedback': feedback,
        'site': Site.objects.get_current()
    }

    html = render_to_string(email_template, context)

    mail_managers(subject=subject, message='', html_message=html)

    _send_sms('contacts/feedback/sms.txt', context)


def send_new_error_message_notification(error_message):

    subject = _('New error message #') + str(error_message.id)

    email_template = 'contacts/error_message/email.html'

    context = {
        'error_message': error_message,
        'site': Site.objects.get_current()
    }

    html = render_to_string(email_template, context)

    mail_managers(subject=subject, message='', html_message=html)

    _send_sms('contacts/error_message/sms.txt', context)


def send_new_return_call_notification(return_call):

    subject = _('New return call request #') + str(return_call.id)

    email_template = 'contacts/return_call/email.html'

    context = {
        'return_call': return_call,
        'site': Site.objects.get_current()
    }

    html = render_to_string(email_template, context)

    mail_managers(subject=subject, message='', html_message=html)

    _send_sms('contacts/return_call/sms.txt', context)




from django import forms

from captcha.fields import ReCaptchaField

from contacts.models import Feedback, ReturnCall, ErrorMessage


class FeedbackForm(forms.ModelForm):

    captcha = ReCaptchaField()

    class Meta:
        model = Feedback
        fields = ('name', 'mobile', 'email', 'subject', 'text', )


class ReturnCallForm(forms.ModelForm):

    class Meta:
        model = ReturnCall
        fields = (
            'answer_time', 'answer_start_time', 'answer_end_time', 'comment',
            'mobile',
        )
        widgets = {
            'answer_time': forms.RadioSelect
        }


class ErrorMessageForm(forms.ModelForm):

    captcha = ReCaptchaField()

    class Meta:
        model = ErrorMessage
        fields = ('text', 'name', 'email', 'mobile', )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2017-07-11 10:47
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contacts', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ErrorMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('mobile', models.CharField(blank=True, max_length=255, null=True, verbose_name='Mobile')),
                ('email', models.EmailField(blank=True, max_length=255, null=True, verbose_name='Email')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('text', models.TextField(max_length=1000, verbose_name='Message text')),
                ('url', models.URLField(max_length=255, verbose_name='URL')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterField(
            model_name='feedback',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]




from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.conf import settings
from django.db import models


class AbstractMessage(models.Model):

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_('User'), blank=True,
        null=True)

    name = models.CharField(_("Name"), max_length=255)

    mobile = models.CharField(
        _("Mobile"), max_length=255, blank=True, null=True)

    email = models.EmailField(
        _("Email"), max_length=255, blank=True, null=True)

    date_created = models.DateTimeField(
        _('Date created'), auto_now_add=True, editable=False)

    text = models.TextField(_('Message text'), max_length=1000, blank=False)

    def __unicode__(self):
        return self.name

    class Meta:
        abstract = True


class Feedback(AbstractMessage):

    subject = models.CharField(_("Subject"), max_length=255)

    class Meta:
        verbose_name = _('Feedback message')
        verbose_name_plural = _('Feedback messages')


class ErrorMessage(AbstractMessage):

    url = models.URLField(_('URL'), max_length=255)

    class Meta:
        verbose_name = _('Error message')
        verbose_name_plural = _('Error messages')


class ReturnCall(models.Model):

    ANSWER_TIME_FAST = 1
    ANSWER_TIME_FIXED = 2

    ANSWER_TIME_CHOICES = (
        (ANSWER_TIME_FAST, _('As fast as possible')),
        (ANSWER_TIME_FIXED, _('In fixed time')),
    )

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, related_name='return_calls',
        verbose_name=_('User'), blank=True, null=True)

    answer_time = models.PositiveIntegerField(
        _('Answer time'), null=False, choices=ANSWER_TIME_CHOICES,
        default=ANSWER_TIME_FAST)

    answer_start_time = models.TimeField(
        _('Start time'), blank=True, null=True, default='08:00')

    answer_end_time = models.TimeField(
        _('End time'), blank=True, null=True, default='08:30')

    mobile = models.CharField(_("Mobile"), max_length=21, blank=False)

    date_created = models.DateTimeField(
        _('Date created'), auto_now_add=True, editable=False)

    comment = models.TextField(_('Comment'), max_length=1000, blank=True)

    def clean_fields(self, exclude=None):

        if self.answer_time != self.ANSWER_TIME_FIXED:
            self.answer_start_time = None
            self.answer_end_time = None
            return

        errors = {}

        start_time = self.answer_start_time
        end_time = self.answer_end_time

        if not start_time:
            errors['answer_start_time'] = start_time.error_messages['required']

        if not end_time:
            errors['answer_end_time'] = end_time.error_messages['required']

        if errors:
            raise ValidationError(errors)

    @property
    def is_time_fixed(self):
        return self.answer_time == self.ANSWER_TIME_FIXED

    def __unicode__(self):
        return self.mobile

    class Meta:
        verbose_name = _('Return call')
        verbose_name_plural = _('Return calls')




from django.test import TestCase
from django.conf import settings


LANGUAGE_CODE = settings.LANGUAGE_CODE


class ContactTestCase(TestCase):

    def test_index_view(self):

        url = '/{}/contacts/'.format(LANGUAGE_CODE)

        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)